#! /usr/bin/perl -w

use DBI;
use strict;

# Configuration
my $sendmail = '|/usr/lib/sendmail -t';
my $address = 'Mailforum <comments@mydomain.tld>';
my $realname = 'The Webmaster';
my $db_name = 'db_name';
my $db_host;
my $db_user = 'db_user';
my $db_pass = 'db_pass';

my ($subject, $parent, $title, $from, $name, $reply_to, $body);

# Parse message
while(<STDIN>) {
	if(/^Subject: (.*)/) {
		$subject = $1;
		if($subject =~ /^\((\d+)\) (.*)/) {
			$parent = $1;
			$title = $2;
		}
	} elsif(/^From: (.*)/) {
		$from = $1;
		if($from =~ /"([^"]*)"/) { $name = $1; }
		elsif($from =~ /([^<@]+) <.*>/) { $name = $1; }
	} elsif(/^Reply-To: (.*)/) {
		$reply_to = $1;
	}
	last if $_ eq "\n";
}

# Fill in undefined variables
$reply_to = $from if !defined($reply_to) && defined($from);

if(!defined($parent) || $parent !~ /^\d+$/) {
	# Subject line was not present or did not have the right format
	exit unless defined($from);

	$name = 'poster' unless defined($name);
	open SENDMAIL, $sendmail;
	print SENDMAIL <<EOT;
To: $reply_to
From: $address
Subject: ERROR: Missing or invalid subject line

Dear $name,

Your comment could not be posted because it did not have a valid subject line. The correct format for the subject line is:

(<parent>) <title>

where <id> is the number of the post you are commenting on, and <title> is the title of your comment. For example:

(1234) I agree

would post a reply with the title "I agree" to the post with number 1234.

Please resend your comment using the correct format for the subject line.

Sincerely,

$realname
EOT
	close SENDMAIL;
	exit;
}

# Quote a string for use in SQL queries
sub sqlize {
	my $ret = $_[0];
	$ret =~ s/(['"])/\\$1/g;
	return '\'' . $ret . '\'';
}

# Prepare variables
$name = (defined($name) ? sqlize($name) : 'NULL');
$title = (defined($title) ? sqlize($title) : 'NULL');
$from = sqlize($from);
$body = '';
while(<STDIN>) { $body .= $_; }
$body = sqlize($body);

# Post comment
my $query = "INSERT INTO Comment VALUES (NULL, $title, $name, $from, $parent, $body);";

my $dbh = DBI->connect("dbi:mysql:$db_name"
	. (defined($db_host) ? ":$db_host" : '') , 'inglorion', 'mvagjKjt') ||
		die('Error connecting to database');

my $res = $dbh->do($query);

$dbh->disconnect();
